/*
 * dstemp.c
 *
 * Created: 2012-03-15 08:12:33
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "defs.h"
#include "dstemp.h"

//#define ONEW_PORT			PORTB
//#define ONEW_DDR			DDRB
//#define ONEW_PIN			PINB
//#define ONEW				2
#define ONEW_H				(ONEW_PORT |= (1<<ONEW))
#define ONEW_L				(ONEW_PORT &= ~(1<<ONEW))
#define ONEW_SETO			(ONEW_DDR |= (1<<ONEW))
#define ONEW_SETI			(ONEW_DDR &= ~(1<<ONEW))
#define ONEW_GET			(ONEW_PIN & (1<<ONEW))

/*#define DS_IDLE				0
#define DS_OK				0
#define DS_CONVERTING		1
#define DS_READY			2
#define DS_ERR				3
#define DS_FAILURE			4
*/

#define DS_ERR_MAX			6
#define DS_OVERTEMP			10000

#define DS_READ_ROM			0x33
#define DS_SKIP_ROM			0xCC
#define DS_START_CONV		0x44
#define DS_RAED_RESULT		0xBE

#define DS_TYPE_NOREAD		0
#define	DS_TYPE_18B20		0x28
#define	DS_TYPE_18S20		0x10


struct {
	char status;
	char type;
	char errors;
	int16_t data;
	int16_t temperature;
} ds;


//-------------------------------------------------------
//simple delay
void SystemUsDelay(char us)
{
	//TCCR0B = 0;
	//TCCR0A = 0;
	SFIOR = (1<<PSR10);
	TIFR |= (1<<TOV0);
	
	us -= 1;
	TCNT0 = (0xFF - us);
	TCCR0 = (1<<CS01);
	
	while((TIFR & (1<<TOV0)) == 0);
	
	/*
	do 
	{
		//_delay_us(1);
		
		//for 8MHz
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		//asm("nop");
		//asm("nop");
		
	} while(--us != 0);
	*/
}
//-------------------------------------------------------
int16_t DSTempGet(void)
{
	return ds.temperature;
}

/*//-------------------------------------------------------
void DSprocessReset(void)
{
	ds.status = DS_IDLE;
	DSGetType();
}*/

//-------------------------------------------------------
void DSprocessInit(void)
{
	ds.status = DS_IDLE;
	ds.type = DS_TYPE_NOREAD;
	ds.errors = DS_ERR_MAX;
}

//------------------------------------
//must be execute about 1/1s, return = 2 then result ready
char DSProcess(void)
{
	if (ds.status >  DS_READY) ds.type = DS_TYPE_NOREAD;	
	if (ds.status >= DS_READY) ds.status = DS_IDLE;

	if (ds.status == DS_IDLE)
	{
		if (ds.type == DS_TYPE_NOREAD)
			DSGetType();
		//else
		{
			if (DSReset() == DS_OK)
			{
				DSWriteByte(DS_SKIP_ROM);		//skip ROM
				DSWriteByte(DS_START_CONV);		//convert
				//power for convert
				ONEW_H;
				ONEW_SETO;
				ds.status = DS_CONVERTING;
			}
			else
				ds.status = DS_ERR;
		}
	}
	else
	{
		if (ds.status == DS_CONVERTING)
		{
			if (DSReset() == DS_OK)
			{
				DSWriteByte(DS_SKIP_ROM);		//skip ROM
				DSWriteByte(DS_RAED_RESULT);		//read
				ds.data = DSReadByte();
				ds.data |= ((DSReadByte())<<8);

				if (TempConvert() == DS_READY)
				{
					ds.status = DS_READY;
					ds.errors = 0;
				}	
				else
					ds.status = DS_ERR;
			}
			else
				ds.status = DS_ERR;
		}
	}
	
	if (ds.status == DS_ERR)
	{
		if (ds.errors < DS_ERR_MAX)	ds.errors++;
	}
	if (ds.errors >= DS_ERR_MAX)	
	{
		return DS_FAILURE;	
	}

	return ds.status;
}

//-------------------------------------
char DSReset (void)
{
	char status;

	status = DS_ERR;

	ONEW_L;
	ONEW_SETO;
	//SystemUsDelay(800);
	SystemUsDelay(255);SystemUsDelay(255);SystemUsDelay(255);

		ONEW_SETI;
		ONEW_H;
		SystemUsDelay(100);

		if (ONEW_GET == 0)
		{
			//SystemUsDelay(600);
			SystemUsDelay(200);SystemUsDelay(200);SystemUsDelay(200);
			if (ONEW_GET != 0) status = DS_OK;
			//else - short to GND
		}
		//else - no DS

	if (status == DS_ERR) ds.status = DS_ERR;
	
	return status;
}

//-------------------------------------
char DSGetType (void)
{
	char i;

	ds.type = DS_TYPE_NOREAD;

	if (DSReset() == DS_OK)
	{
		DSWriteByte(DS_READ_ROM);				//read ROM
		i = DSReadByte();

		if ((i == DS_TYPE_18B20) || (i == DS_TYPE_18S20))
			ds.type = i;
	}
	
	return ds.type;
}

//-------------------------------------
void DSWriteByte(char byte)
{
	char i;
	for (i=0; i<8; i++)
	{
		DSWriteBit(byte & 0x01);
		byte >>= 1;
	}
}

//-------------------------------------
void DSWriteBit(char bit)
{
	ONEW_L;
	ONEW_SETO;
	SystemUsDelay(6);

	if (bit > 0) ONEW_H;

	SystemUsDelay(64);

	ONEW_H;
	SystemUsDelay(20);
}

//-------------------------------------
char DSReadByte(void)
{
	char i;
	char temp = 0;

	for (i=0; i<8; i++)
	{
		temp >>= 1;
		if (DSReadBit() > 0) temp |= 0x80;
	}
	return temp;
}

//-------------------------------------
char DSReadBit(void)
{
	char state = 0;

	ONEW_L;
	ONEW_SETO;
	SystemUsDelay(6);
	ONEW_SETI;
	ONEW_H;	
	
	SystemUsDelay(6);
	if (ONEW_GET != 0) state = 1;

	SystemUsDelay(78);
	return state;
}

//-------------------------------------
int16_t TempConvert(void)
{
	int32_t buffer = 0;
	char status = DS_ERR;

	if (ds.type)
	{
		if (ds.type == DS_TYPE_18B20)
		{
			buffer = (int32_t)(ds.data);
			buffer *= 625;
			buffer /= 1000;
		}

		if (ds.type == DS_TYPE_18S20)
		{
			buffer = (ds.data * 5);
		}

		if ((buffer <=  1250) && (buffer >= -550))
		{
			ds.temperature = (int16_t)buffer;
			status = DS_READY;
		}
	}
	return status;
}
