/*
 * termostat_5354.c
 *
 * Created: 2015-09-07 07:15:18
 *  Author: dsosnowski
 */ 

/*
 * dtc.c
 *
 * Created: 2015-02-19 15:09:48
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "lcd.h"
#include "wdog.h"
#include "dstemp.h"
#include "ee_mem.h"


void HardInit(void);
char UiKbdScan(void);
void OutMan(char status);


struct {
	char level;
	char press;
	char cycle;
	char refresh;
} system;

struct {
	int16_t ton;
	int16_t toff;
	int16_t mode;
	char disable;
	char action;
	char store;
	char idle;
	char style;
} settings;

extern struct {
	char status;
	char type;
	char errors;
	int16_t data;
	int16_t temperature;
} ds;

//--------------------------------------------------------------
int main(void)
{
	HardInit();
	//Executive Airflow
	LCDInit();
	LCDLocate(0,0);
	//LCDText("   Termostat    ");
	//LCDLocate(1,0);
	//LCDText("    AVT5354     ");
	LCDText("   Executive    ");
	LCDLocate(1,0);
	LCDText("    Airflow     ");
	_delay_ms(500);
	settings.style = MODE_CHECK;
    DSprocessInit();
	
	settings.ton = EEread16(4);	
	settings.toff = EEread16(6);
	settings.mode = EEread16(8);

	if ((settings.ton > TEMP_MAX_RANGE) && (settings.ton < TEMP_MIN_RANGE)) settings.ton = 200;
	if ((settings.toff > TEMP_MAX_RANGE) && (settings.toff < TEMP_MIN_RANGE)) settings.toff = 220;
	if (settings.ton < settings.toff)settings.ton = settings.toff;
	if ((settings.mode > 1) && (settings.mode < 0)) settings.mode = 0;
	
	while(1)
    {
		//------------------------------------------------------
		if (UiKbdScan())
		{			
			if (system.press == 3) 
			{
				system.level++;
			}
			if (system.level >  3) system.level = 0;

			if (system.level == 0)
			{
				if (system.press == 4)
				{
					if (settings.disable == 1) OutMan(3);
				}

				if (system.press == (4 + BUTTON_HOLD))
				{
					if (settings.disable == 0) 
					{
						settings.disable = 1;
						LED_ON;
					}
					else
					{
						settings.disable = 0;					
						LED_OFF;
					}
				}
			}
			
			if (system.level == 1)
			{
				if (system.press == 1) 
				{
					settings.ton++;
					settings.store = 5;
				}
				if (system.press == 2) 
				{
					settings.ton--;
					settings.store = 5;
				}
				if (system.press == (1 | BUTTON_HOLD)) settings.ton += 10;
				if (system.press == (2 | BUTTON_HOLD)) settings.ton -= 10;
							
				if (settings.style == 0)
				{				
					if (settings.ton < TEMP_MIN_RANGE) 
					{
						settings.ton = TEMP_MIN_RANGE;
					}
					if (settings.ton > TEMP_MAX_RANGE) 
					{
						settings.ton = TEMP_MAX_RANGE;
					}
					if (settings.ton < settings.toff) settings.toff = settings.ton;
				}
				else
				{
					if (settings.ton < TEMP_MIN_RANGE)
					{
						settings.ton = TEMP_MIN_RANGE;
					}
					if (settings.ton > TEMP_MAX_RANGE)
					{
						settings.ton = TEMP_MAX_RANGE;
					}					
				}
			}
				
			if (system.level == 2)
			{
				
				if (system.press == 1) 
				{
					settings.toff++;
					settings.store = 5;
				}
				if (system.press == 2) 
				{
					settings.toff--;
					settings.store = 5;
				}
				if (system.press == (1 | BUTTON_HOLD)) settings.toff += 10;
				if (system.press == (2 | BUTTON_HOLD)) settings.toff -= 10;

				if (settings.style == 0)
				{
					if (settings.toff < TEMP_MIN_RANGE) 
					{
						settings.toff = TEMP_MIN_RANGE;
					}
					if (settings.toff > TEMP_MAX_RANGE) 
					{
						settings.toff = TEMP_MAX_RANGE;
					}
					if (settings.toff > settings.ton) settings.ton = settings.toff;
				}
				else
				{
					if (settings.toff < 0)
					{
						settings.toff = 0;
					}
					if (settings.toff > 100)
					{
						settings.toff = 100;
					}
				}
				
			}

			if (system.level == 3)
			{
				
				if (system.press == 1)
				{
					settings.mode = 0;
					settings.store = 5;
				}
				if (system.press == 2)
				{
					settings.mode  = 1;
					settings.store = 5;
				}
			}

			settings.idle = 15;
			system.refresh = 1;
		}
		
		//------------------------------------------------------
		if (system.cycle > 38)
		{
			system.cycle = DSProcess();	
			if (system.cycle == DS_READY) system.refresh = 1;
			if (system.cycle == DS_FAILURE) system.refresh = 2;
			
			if ((settings.store) && (system.level == 0))
			{
				settings.store--;
				if (settings.store == 0)
				{
					EEwrite16(4, settings.ton);
					EEwrite16(6, settings.toff);
					EEwrite16(8, settings.mode);
				}	
			}
			
			if (settings.idle)
			{
				settings.idle--;
				if (settings.idle == 0) system.level = 0;			
			}
			
			system.cycle = 0;
		}

		//------------------------------------------------------
		if (settings.disable == 0)
			OutMan(system.refresh);
				
		//------------------------------------------------------
		if (system.refresh)
		{			
			if (system.level == 0)
			{
				LCDLocate(0,0);
				LCDText(" Temp: ");
				LCDTempValue(ds.temperature);
				LCDData(0); LCDText("C  ");
		
				if (system.refresh == 1)
				{
					/*if (settings.ton <= settings.toff)
					{
						if (OUTPUT_GET == 0) LCDData(0);
						else LCDData(1);
					}
					else
					{	
						if (OUTPUT_GET == 0) LCDData(2);
						else LCDData(3);
					}
					
					LCDTempValue(ds.temperature);
					*/
				}
				
				if (system.refresh == 2)
				{
					LCDLocate(0,0);
					LCDText(" Temp: ---.-    ");
				}
				
				if (settings.disable == 0)
				{
					LCDLocate(1,0);
					if (settings.style == 0)
						LCDData('G');
					else
						LCDData('U');
					LCDTempValue(settings.ton);
				
					//LCDText("    ");
					LCDData(' ');
					LCDData(1);
					LCDData(1);
					LCDData(' ');
				
				
					if (settings.style == 0)
						LCDData('D');
					else
						LCDData('H');
					LCDTempValue(settings.toff);				
				}
				else
				{
					LCDLocate(1,0);
					LCDText(" Ster. manualne ");
				}
			}
			if (system.level == 1)
			{
				LCDLocate(1,0);
				if (settings.style == 0)
					LCDData('G');
				else
					LCDData('U');
				LCDTempValue(settings.ton);
				
				LCDText(" << Ustaw ");
			}

			if (system.level == 2)
			{
				LCDLocate(1,0);
				LCDText(" Ustaw >> ");

				if (settings.style == 0)
					LCDData('D');
				else
					LCDData('H');
				LCDTempValue(settings.toff);
			}
			
			if (system.level == 3)
			{
				LCDLocate(1,0);
				LCDText("Ustaw>>");
				LCDData(1);
				LCDData(1);				
				LCDText("<<     ");
			}
		}
		
		//------------------------------------------------------
		if (settings.mode == 0)
			LCDAnimation(0);
		else
			LCDAnimation(1);
		
		//------------------------------------------------------
		system.refresh = 0;
		_delay_ms(MAIN_DELAY);
		system.cycle++;
		WdtReset();
    }	
}


//--------------------------------------------------------------
void HardInit(void)
{
	OUT_PORT &= ~(1<<OUTPUT);
	OUT_PORT |=  (1<<LED_PIN);
	OUT_DDR |= (1<<OUTPUT | 1<<LED_PIN);

	SW_PORT |= (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN | 1<<SW4_PIN);
	
	ONEW_PORT |= (1<<ONEW);
	
	cli();
	WdtSet(7);
}

//--------------------------------------------------------------
void OutMan(char status)
{
	
		if (status == 1)		//normal
		{
			LED_OFF;
			
			if (settings.style == 0)				//min max
			{
				if (settings.mode == 0)		//grzanie
				{
					if (ds.temperature >= settings.ton) OUTPUT_OFF;
					if (ds.temperature <= settings.toff) OUTPUT_ON;					
				}
				else						//chodzenie
				{
					if (ds.temperature >= settings.ton) OUTPUT_ON;
					if (ds.temperature <= settings.toff) OUTPUT_OFF;
				}
			}
			
			else			//hystereza
			{
				if (settings.mode == 0)		//grzanie
				{
					if (ds.temperature >= (settings.ton + settings.toff)) OUTPUT_OFF;
					if (ds.temperature <= (settings.ton - settings.toff)) OUTPUT_ON;
				}
				else						//chodzenie
				{
					if (ds.temperature >= (settings.ton + settings.toff)) OUTPUT_ON;
					if (ds.temperature <= (settings.ton - settings.toff)) OUTPUT_OFF;
				}				
			}
		}
	
		if (status == 2)		//DS error
		{
			LED_OFF;
			OUTPUT_OFF;
		}

		if (status == 3)		//manual
		{
			LED_ON;
			
			if (OUTPUT_GET == 0)
				OUTPUT_ON;
			else
				OUTPUT_OFF;	
		}
}

//----------------------------------------------
char UiKbdScan(void)
{
	char button;
	static char prev;
	static char hold;

	button = 0;
	if ((GET_SW & (1<<SW3_PIN)) == 0) button = 1;
	if ((GET_SW & (1<<SW2_PIN)) == 0) button = 2;
	if ((GET_SW & (1<<SW4_PIN)) == 0) button = 3;
	if ((GET_SW & (1<<SW1_PIN)) == 0) button = 4;

	//any state
	if (button == 0)
	{
		if (prev)
		{
			if ((hold >= BUTTON_CLICK) && (hold < BUTTON_THRESHOLD - BUTTON_REPEAT))
			button = prev;
		}
		prev = 0;			//sw_free
		hold = 0;
	}
	else
	{
		if (hold < 0xFF) hold++;		//counting hold time
		if (button != prev) hold = 0;	//new key - reset counters

		prev = button;
		button = 0;

		if (hold == BUTTON_THRESHOLD) //after threshold - repeat key
		{
			//auto repeat PLUS & MINUS key
			if ((prev == 1) || (prev == 2))
			{
				hold -= BUTTON_REPEAT;
				//button = (prev | BUTTON_HOLD);
				//button = prev;
			}
			button = (prev | BUTTON_HOLD);
		}
	}
	system.press = button;
	return button;
}


